package aggtool.ui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import com.microsoft.z3.BoolExpr;
import com.microsoft.z3.Expr;
import com.microsoft.z3.Solver;
import com.microsoft.z3.Status;

import aggtool.metadata.DAPFormulas;
import aggtool.z3.Z3Analyzer;

public class CheckPanel extends JPanel {

	public Z3Analyzer z3analyzer;
	public JButton btCheckDAPSpecification = new JButton("Check DAP Specifications");
	public JButton btCheckDBMSSelection = new JButton("Check DBMS Selection");
	public JTextArea txtCheckDAP = new JTextArea();
	public JScrollPane spCheckDAP = new JScrollPane(txtCheckDAP);
	public JTextArea txtCheckDBMS = new JTextArea();
	public JScrollPane spCheckDBMS = new JScrollPane(txtCheckDBMS);
	public NewRulePanel pLoadRule;
	
	
	public CheckPanel(Z3Analyzer z) {
		super();
		z3analyzer = z;
		initializer();
	}
	
	void initializer() {
		this.setLayout(new BorderLayout());
		
		pLoadRule = new NewRulePanel(z3analyzer);
		this.add(pLoadRule, BorderLayout.NORTH);
		
		JPanel pDAPCheck = new JPanel(new BorderLayout());
		JPanel pbutton1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pbutton1.add(btCheckDAPSpecification);
		txtCheckDAP.setEditable(false);
		pDAPCheck.add(pbutton1, BorderLayout.NORTH);
		pDAPCheck.add(spCheckDAP, BorderLayout.CENTER);
		

		JPanel pDBMSCheck = new JPanel(new BorderLayout());
		JPanel pbutton2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pbutton2.add(btCheckDBMSSelection);
		txtCheckDBMS.setEditable(false);
		pDBMSCheck.add(pbutton2, BorderLayout.NORTH);
		pDBMSCheck.add(spCheckDBMS, BorderLayout.CENTER);
		
		this.add(pDAPCheck, BorderLayout.CENTER);
		//this.add(pDBMSCheck);
		
		implementRules();
	}
	
	void  implementRules(){
		btCheckDAPSpecification.addActionListener(new ActionListener()
		{
			  public void actionPerformed(ActionEvent e)
			  {
			    // display/center the jdialog when the button is pressed
				  checDAPSpec();
			  }
			});
		
		btCheckDBMSSelection.addActionListener(new ActionListener()
		{
			  public void actionPerformed(ActionEvent e)
			  {
			    // display/center the jdialog when the button is pressed
				  checDBMSSelection();
			  }
			});
	}
	
	void checDAPSpec() {
		Solver solver = z3analyzer.z3context.mkSolver();
		
		ArrayList<BoolExpr> assumptions = new ArrayList<BoolExpr> ();
		for(DAPFormulas formulas : z3analyzer.DAPFormulaList) {
			for(BoolExpr f : formulas.formulas)
				assumptions.add(f);
		}
		BoolExpr[] assumptions_to_check = assumptions.toArray(new BoolExpr[0]);
		for(BoolExpr c : z3analyzer.IntraDAPConstraints) {
			solver.add(c);
		}
		for(BoolExpr c : z3analyzer.InterDAPConstraints) {
			solver.add(c);
		}
		for(BoolExpr c : z3analyzer.CustomizedConstraints) {
			solver.add(c);
		}
		
		Status status = solver.check(assumptions_to_check);
		txtCheckDAP.setText("");
		String result = "";
		if (status == Status.SATISFIABLE) {
            System.out.println( "DAP specifications consistency check passed!");
            result = "Consistency check powered by Microsoft Z3 Theorem Prover: DAP specification consistency verified!";
			txtCheckDAP.setText(result);
		} else {

        	System.out.println( "DAP specifications consistency check failed!");
        	result = "Consistency check performed by Microsoft Z3 Theorem Prover: DAP specification consistency check failed!\n";

        	result += "Inconsistent specifications that violate design constraints are: \n";
			int size = solver.getUnsatCore().length;
			int i = 0;
             for (Expr c : solver.getUnsatCore())
             {
            	 result = result + "(" + c + ") ";
            	 if(i<size-1) result += "and ";
            	 i++;
             }
        	 txtCheckDAP.append(result);
        }
		

		txtCheckDAP.append("\n\n======================Detailed Log=====================\n\n");		
		txtCheckDAP.append("The consistency check is performed by Z3 on the conjuction of logical formulas transformed from the selected features in theDAP specifications, the constraints on the features within each DAP, and the constraints on features between dependent DAP.\n");
		txtCheckDAP.append("\nLogical formulas transformed from the selected features in the DAP specifications:\n");
		for(DAPFormulas formulas : z3analyzer.DAPFormulaList) {
			for(BoolExpr f : formulas.formulas)
				txtCheckDAP.append(f.toString()+"\n");
		}
		
		txtCheckDAP.append("\nConstraints on the features within each DAP:\n");
		for(BoolExpr c : z3analyzer.IntraDAPConstraints) {
			txtCheckDAP.append(c.toString()+"\n");
		}
		
		txtCheckDAP.append("\nConstraints on the features between dependent DAP:\n");
		if(z3analyzer.InterDAPConstraints.size() == 0) txtCheckDAP.append("None.\n");
		for(BoolExpr c : z3analyzer.InterDAPConstraints) {
			txtCheckDAP.append(c.toString()+"\n");
		}
		
		txtCheckDAP.append("\nCustomized constraints added by user:\n");
		if(z3analyzer.CustomizedConstraints.size() == 0) txtCheckDAP.append("None.\n");
		for(BoolExpr c : z3analyzer.CustomizedConstraints) {
			txtCheckDAP.append(c.toString()+"\n");
		}

		txtCheckDAP.append("\n" + result);
		
	}
	
	void checDBMSSelection() {
//		Solver solver = z3analyzer.z3context.mkSolver();
//		
//		ArrayList<BoolExpr> assumptions = new ArrayList<BoolExpr> ();
//		for(DAPFormulas formulas : z3analyzer.DAPFormulaList) {
//			for(BoolExpr f : formulas.formulas)
//				assumptions.add(f);
//		}
//		BoolExpr[] assumptions_to_check = assumptions.toArray(new BoolExpr[0]);
//		for(BoolExpr c : z3analyzer.IntraDAPConstraints) {
//			solver.add(c);
//		}
//		
//		Status status = solver.check(assumptions_to_check);
//		txtCheckDAP.setText("");
//		if (status == Status.SATISFIABLE) {
//            System.out.println( "DAP specifications consistency check passed!");
//			txtCheckDAP.setText("DAP specifications consistency check passed!");
//		} else {
//
//        	System.out.println( "DAP specifications consistency check failed!");
//			txtCheckDAP.append("DAP specifications consistency check failed!\n");
//        	// System.out.println("proof: " + solver.getProof());
//			txtCheckDAP.append("Specifications violating the check are: \n");
//             for (Expr c : solver.getUnsatCore())
//             {
//            	 txtCheckDAP.append(c + "\n");
//             }
//        }
	}
}
