package aggtool.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import aggtool.metadata.DAPFormulas;
import aggtool.metadata.DAPSpec;
import aggtool.z3.Z3Analyzer;
import aggtool.z3.Z3Utils;

import com.microsoft.z3.BoolExpr;
import com.microsoft.z3.Context;
import com.microsoft.z3.Expr;
import com.microsoft.z3.Solver;
import com.microsoft.z3.Status;

public class DAPListPanel extends JPanel {

	public ArrayList<DAPPanel> dappanels = new ArrayList<DAPPanel>();
	public JButton btAddDAP = new JButton("New Data Aggregation Process");
	public JButton btClear = new JButton("Clear All");
	public JPanel daplistpanel =  new JPanel();
	public JScrollPane scrollPane = new JScrollPane();
	public InterDAPDepenencyUpperPanel dapdeppanel;

	//public JButton btCheck = new JButton("Consistency Check");
	public JButton btLoad = new JButton("Load");
	public Z3Analyzer z3analyzer;

	public DAPListPanel(Z3Analyzer z) {
		super();
		z3analyzer = z;
		initialize();
	}
	
	void initialize() {

		//this.setLayout(new FlowLayout(FlowLayout.LEFT));
		this.setLayout(new BorderLayout());
		
		JPanel panel = new JPanel(new BorderLayout());
		//panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		
		JPanel pbutton = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pbutton.add(btAddDAP);
		pbutton.add(btClear);
		panel.add(pbutton, BorderLayout.NORTH);
		
		//DAPPanel dappanel1 = new DAPPanel();
		//dappanel1.txtName.setText("DAP"+(dappanels.size()+1));
		//dappanels.add(dappanel1);
		daplistpanel.setLayout(new BoxLayout(daplistpanel, BoxLayout.Y_AXIS));
		//daplistpanel.add(dappanel1);

		daplistpanel.setBorder(BorderFactory.createLoweredBevelBorder());
		
		scrollPane.setViewportView(daplistpanel);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		
		panel.add(daplistpanel, BorderLayout.CENTER);
		
		

		JPanel pbutton2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pbutton2.add(btLoad);
//		pbutton2.add(btCheck);
		panel.add(pbutton2, BorderLayout.SOUTH);
		
		//this.add(panel);
		dapdeppanel = new InterDAPDepenencyUpperPanel(z3analyzer);
		JPanel panel2 = new JPanel(new BorderLayout());
		panel2.add(dapdeppanel, BorderLayout.CENTER);

		//this.add(dapdeppanel);
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				panel, panel2);
		splitPane.setDividerLocation(600);
		//this.setMaximumSize(new Dimension(700, 400));
		this.add(splitPane, BorderLayout.CENTER);
		implementRules();
	}
	
	void implementRules() {
		btAddDAP.addActionListener(new ActionListener()
		{
			  public void actionPerformed(ActionEvent e)
			  {
			    // display/center the jdialog when the button is pressed
				  addNewDAP();
			  }
			});
		
		btLoad.addActionListener(new ActionListener()
		{
			  public void actionPerformed(ActionEvent e)
			  {
			    // display/center the jdialog when the button is pressed
				  loadDAP();
			  }
			});
		
		btClear.addActionListener(new ActionListener()
		{
			  public void actionPerformed(ActionEvent e)
			  {
				  clearAll();
			  }
			});
		
		
		
//		btCheck.addActionListener(new ActionListener()
//		{
//			  public void actionPerformed(ActionEvent e)
//			  {
//			    // display/center the jdialog when the button is pressed
//				  check();
//			  }
//			});
	}
	public void clearAll() {

		z3analyzer.DAPFormulaList.clear();
		z3analyzer.DAPList.clear();
		z3analyzer.InterDAPConstraints.clear();
		z3analyzer.IntraDAPConstraints.clear();
		z3analyzer.CustomizedConstraints.clear();
		dappanels.clear();
		daplistpanel.removeAll();
		DAPPanel dappanel1 = new DAPPanel();
		dappanel1.txtName.setText("DAP"+(dappanels.size()+1));
		dappanels.add(dappanel1);
		daplistpanel.add(dappanel1);
		
		dapdeppanel.clearAll();
		this.revalidate();
		this.repaint();
	}
	
	public void addNewDAP() {
		DAPPanel dappanel = new DAPPanel();
		dappanel.txtName.setText("DAP"+(dappanels.size()+1));
		dappanels.add(dappanel);
		daplistpanel.add(dappanel);
		this.revalidate();
		this.repaint();
	}
	
	public void loadDAP() {
		z3analyzer.DAPFormulaList.clear();
		z3analyzer.DAPList.clear();
		for(DAPPanel dap : dappanels) {
			DAPFormulas formulas = new DAPFormulas();
			createFormulasForDAP(dap, formulas);
			z3analyzer.DAPFormulaList.add(formulas);
			z3analyzer.DAPList.addElement(DAPSpec.createDAPSpecFromPanel(dap));
		}
		//z3analyzer.DAPList.notify();
		for(DAPFormulas fs : z3analyzer.DAPFormulaList) {
			fs.print();
		}
		System.out.println("DAP Specs loaded!");
		z3analyzer.addIntraDAPRules(dappanels);
		for(BoolExpr f : z3analyzer.IntraDAPConstraints) {
			System.out.println(f.toString());
		}
	}
	
	public void createFormulasForDAP(DAPPanel dap, DAPFormulas formulas) {
		Context z3context = z3analyzer.z3context;
		String DAPName = dap.txtName.getText();
		formulas.addFormula(Z3Utils.createBooleanFormula(z3context, DAPName, true));
		
		for(RDTypePanel rdt: dap.rdtypepanels) {
			String rdtname = rdt.txtName.getText();
			String rdtvariable = DAPName + "_" + rdtname;
			formulas.addFormula(Z3Utils.createBooleanFormula(z3context, rdtvariable, true));
			for(RawDataPanel rd: rdt.rdpanels) {
				String rdname = rd.txtName.getText();
				String rdvariable = rdtvariable + "_" + rdname;
				formulas.addFormula(Z3Utils.createBooleanFormula(z3context, rdvariable, true));
				
				recursiveCreateFormulas(rd, formulas, rdvariable);
			}
		}
		
		String afvariable = DAPName + "_" + "AggregateFunction";
		formulas.addFormula(Z3Utils.createBooleanFormula(z3context, afvariable, true));
		recursiveCreateFormulas(dap.pAggregateFunction, formulas, afvariable);
		
		String advariable = DAPName + "_" + dap.pAggregatedData.txtName.getText();
		formulas.addFormula(Z3Utils.createBooleanFormula(z3context, advariable, true));
		recursiveCreateFormulas(dap.pAggregatedData, formulas, advariable);

		String tpvariable = DAPName + "_" + "TriggeringPattern";
		formulas.addFormula(Z3Utils.createBooleanFormula(z3context, tpvariable, true));
		recursiveCreateFormulas(dap.pTP, formulas, tpvariable);
		

		String rtpvariable = DAPName + "_" + "RealTimeProcess";
		if(dap.pRTProcess.cbRTPEnable.isSelected())
			formulas.addFormula(Z3Utils.createBooleanFormula(z3context, rtpvariable, true));
		else
			formulas.addFormula(Z3Utils.createBooleanFormula(z3context, rtpvariable, false));
		recursiveCreateFormulas(dap.pRTProcess.pInner, formulas, rtpvariable);
	}
	
	void recursiveCreateFormulas(JPanel panel, DAPFormulas formulas, String prefix) {
		for( Component comp : panel.getComponents() ) {
			   if( comp instanceof JCheckBox) {
				   JCheckBox cb = (JCheckBox) comp;
				   if(cb.isSelected() && cb.isEnabled())  {
					   formulas.addFormula(Z3Utils.createBooleanFormula(z3analyzer.z3context, prefix + "_" + cb.getText(), true));
				   } else {
					   formulas.addFormula(Z3Utils.createBooleanFormula(z3analyzer.z3context, prefix + "_" + cb.getText(), false));
				   }
			   } else if( comp instanceof JRadioButton) {
				   JRadioButton rb = (JRadioButton) comp;
				   if(rb.isSelected() && rb.isEnabled())  {
					   formulas.addFormula(Z3Utils.createBooleanFormula(z3analyzer.z3context, prefix + "_" + rb.getText(), true));
				   } else {
					   formulas.addFormula(Z3Utils.createBooleanFormula(z3analyzer.z3context, prefix + "_" + rb.getText(), false));
				   }
			   } else if( comp instanceof JPanel) {
				   JPanel p = (JPanel) comp;
				   String newprefix = prefix;
				   if(p.getName()!=null && p.getName().length()>0)
					   newprefix = prefix + "_" + p.getName();
				   recursiveCreateFormulas(p, formulas, newprefix);
			   }
		}
	}
	
//	public void check() {
//		Solver solver = z3analyzer.z3context.mkSolver();
//		
//		ArrayList<BoolExpr> assumptions = new ArrayList<BoolExpr> ();
//		for(DAPFormulas formulas : z3analyzer.DAPFormulaList) {
//			for(BoolExpr f : formulas.formulas)
//				assumptions.add(f);
//		}
//		BoolExpr[] assumptions_to_check = assumptions.toArray(new BoolExpr[0]);
//		for(BoolExpr c : z3analyzer.IntraDAPConstraints) {
//			solver.add(c);
//		}
//		
//		Status status = solver.check(assumptions_to_check);
//		if (status == Status.SATISFIABLE)
//            System.out.println( "consistency proved");
//        else {
//
//        	System.out.println( "consistency violated");
//        	// System.out.println("proof: " + solver.getProof());
//             System.out.println("core: ");
//             for (Expr c : solver.getUnsatCore())
//             {
//                 System.out.println(c);
//             }
//        }
//	}
}
