package aggtool.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;

import aggtool.metadata.DAPFormulas;
import aggtool.metadata.DBMSFormulas;
import aggtool.metadata.DBMSSpec;
import aggtool.z3.Z3Analyzer;
import aggtool.z3.Z3Utils;

import com.microsoft.z3.BoolExpr;
import com.microsoft.z3.Context;

public class DBMSListPanel extends JPanel {
	public ArrayList<DBMSPanel> dbmspanels = new ArrayList<DBMSPanel>();
	public JButton btAddDBMS = new JButton("New DBMS");
	public JButton btClear = new JButton("Clear all");
	public JButton btLoad = new JButton("Load all DBMS");
	public JPanel dbmslistpanel = new JPanel();
	public DBMSSelectionUpperPanel dbmsselectionpanel;
	public Z3Analyzer z3analyzer;
	public JScrollPane scrollPane = new JScrollPane();
	
	public DBMSListPanel(Z3Analyzer z) {
		super();
		z3analyzer = z;
		initializer();
	}
	
	void initializer() {
		//this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.setLayout(new BorderLayout());
		JPanel dbmsspecpane = new JPanel(new BorderLayout());
		//this.setLayout(new BorderLayout());


		JPanel pbutton = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pbutton.add(btAddDBMS);
		pbutton.add(btClear);
		dbmsspecpane.add(pbutton, BorderLayout.NORTH);
		
		DBMSPanel dbmspanel1 = new DBMSPanel();
		dbmspanel1.txtName.setText("DBMS"+(dbmspanels.size()+1));
		dbmspanels.add(dbmspanel1);
		dbmslistpanel.setLayout(new BoxLayout(dbmslistpanel, BoxLayout.Y_AXIS));
		dbmslistpanel.add(dbmspanel1);
		//dbmspanel1.setAlignmentY(TOP_ALIGNMENT);
		

		scrollPane.setViewportView(dbmslistpanel);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		dbmsspecpane.add(scrollPane, BorderLayout.CENTER);
		
		JPanel pbutton2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pbutton2.add(btLoad);
		dbmsspecpane.add(pbutton2, BorderLayout.SOUTH);
		//dbmsspecpane.setMinimumSize(new Dimension(1000, 1000));

		dbmsselectionpanel = new DBMSSelectionUpperPanel(z3analyzer);
		

		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				dbmsspecpane, dbmsselectionpanel);
		splitPane.setDividerLocation(0.5);
		this.add(splitPane, BorderLayout.CENTER);
		implementRules();
	}
	
	void implementRules() {
		btAddDBMS.addActionListener(new ActionListener()
		{
			  public void actionPerformed(ActionEvent e)
			  {
			    // display/center the jdialog when the button is pressed
				  addNewDBMS();
			  }
			});
		

		btClear.addActionListener(new ActionListener()
		{
			  public void actionPerformed(ActionEvent e)
			  {
			    // display/center the jdialog when the button is pressed
				  clearAll();
			  }
			});
		
		btLoad.addActionListener(new ActionListener()
		{
			  public void actionPerformed(ActionEvent e)
			  {
			    // display/center the jdialog when the button is pressed
				  loadDBMS();
			  }
			});
	}
	
	public void clearAll() {
		z3analyzer.DBMSList.clear();
		z3analyzer.DBMSFormulaList.clear();
		z3analyzer.DBMSConstraints.clear();
		dbmspanels.clear();
		dbmslistpanel.removeAll();
		DBMSPanel dbmspanel1 = new DBMSPanel();
		dbmspanel1.txtName.setText("DBMS"+(dbmspanels.size()+1));
		dbmspanels.add(dbmspanel1);
		dbmslistpanel.setLayout(new BoxLayout(dbmslistpanel, BoxLayout.Y_AXIS));
		dbmslistpanel.add(dbmspanel1);
		
		dbmsselectionpanel.clearAll();
		this.revalidate();
		this.repaint();
	}
	
	public void addNewDBMS() {
		DBMSPanel dbmspanel = new DBMSPanel();
		dbmspanel.txtName.setText("DBMS"+(dbmspanels.size()+1));
		dbmspanels.add(dbmspanel);
		dbmslistpanel.add(dbmspanel);
		this.revalidate();
		this.repaint();
	}
	
	public void loadDBMS() {
		z3analyzer.DBMSFormulaList.clear();
		z3analyzer.DBMSList.clear();
		for(DBMSPanel dbms : dbmspanels) {
			DBMSFormulas formulas = new DBMSFormulas();
			createFormulasForDBMS(dbms, formulas);
			z3analyzer.DBMSFormulaList.add(formulas);
			z3analyzer.DBMSList.addElement(createDBMSSpec(dbms));
			
		}
		for(DBMSFormulas fs : z3analyzer.DBMSFormulaList) {
			fs.print();
		}
		System.out.println("DBMS Specs loaded!");
		
	}
	
	public void createFormulasForDBMS(DBMSPanel dbms, DBMSFormulas formulas) {
		Context z3context = z3analyzer.z3context;
		String DBMSName = dbms.txtName.getText();
		formulas.addFormula(Z3Utils.createBooleanFormula(z3context, DBMSName, true));
		
		for( Component comp : dbms.pDbmscontentpanel.getComponents() ) {
			   if( comp instanceof JCheckBox) {
				   JCheckBox cb = (JCheckBox) comp;
				   if(cb.isSelected() && cb.isEnabled())  {
					   formulas.addFormula(Z3Utils.createBooleanFormula(z3analyzer.z3context, DBMSName + "_" + cb.getText(), true));
				   } else {
					   formulas.addFormula(Z3Utils.createBooleanFormula(z3analyzer.z3context, DBMSName + "_" + cb.getText(), false));
				   }
			   } 
		}
	}
	
	public DBMSSpec createDBMSSpec(DBMSPanel dbms) {
		String DBMSName = dbms.txtName.getText();
		DBMSSpec dbmsspec = new DBMSSpec(DBMSName, dbms.cbAtomicity.isSelected(), dbms.cbConsistency.isSelected(), 
				dbms.cbIsolation.isSelected(), dbms.cbDurability.isSelected(), dbms.cbTimeliness.isSelected());
		return dbmsspec;
	}
}
