package aggtool.ui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import aggtool.metadata.DAPSpec;
import aggtool.metadata.DAPSpecList;
import aggtool.metadata.DBMSSpec;
import aggtool.metadata.DBMSSpecList;
import aggtool.metadata.RDTypeSpec;
import aggtool.metadata.RawDataSpec;
import aggtool.z3.Z3Analyzer;

public class DBMSSelectionPanel extends JPanel{
	Z3Analyzer z3analyzer;
	JComboBox cbDAP = new JComboBox();
	JComboBox cbDBMS = new JComboBox();
	DAPSpecList daps = new DAPSpecList();
	DBMSSpecList alternatives = new DBMSSpecList();
	JTextArea textArea = new JTextArea();
	
	public DBMSSelectionPanel(Z3Analyzer z) {
		super();

		z3analyzer = z;
		initialize();
		implementRules();
	}
	
	void initialize() {
		this.setLayout(new BorderLayout());
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JLabel lDap = new JLabel("Data aggregation process ");
		panel.add(lDap);
		cbDAP.setModel(z3analyzer.DAPList);
		cbDAP.setPrototypeDisplayValue("Data Aggregation Process");

		panel.add(cbDAP);
		JLabel lDbms = new JLabel(" is managed by DBMS ");
		panel.add(lDbms);
		cbDBMS.setModel(alternatives);
		cbDBMS.setPrototypeDisplayValue("DBMS ABC");
		panel.add(cbDBMS);
		JPanel panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JLabel lexplain = new JLabel("(Only feasible DBMS are listed and selectable)");
		panel2.add(lexplain);
		panel.add(panel2);
		
		this.add(panel, BorderLayout.NORTH);
		
		JPanel modelpanel = new JPanel(new BorderLayout());
		
		textArea.setSize(400, 100);

		textArea.setLineWrap(true);
		textArea.setEditable(false);
		textArea.setVisible(true);

		JScrollPane scroll = new JScrollPane(textArea);
		//scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		//scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		modelpanel.add(textArea, BorderLayout.CENTER);


		this.add(modelpanel, BorderLayout.CENTER);
	}
	
	void implementRules() {
//		cbDAP.addMouseListener(new MouseListener () {
//
//			@Override
//			public void mouseClicked(MouseEvent arg0) {
//				// TODO Auto-generated method stub
//				daps.copy(z3analyzer.DAPList);
//			}
//
//			@Override
//			public void mouseEntered(MouseEvent arg0) {
//				// TODO Auto-generated method stub
//				
//			}
//
//			@Override
//			public void mouseExited(MouseEvent arg0) {
//				// TODO Auto-generated method stub
//				
//			}
//
//			@Override
//			public void mousePressed(MouseEvent arg0) {
//				// TODO Auto-generated method stub
//				
//			}
//
//			@Override
//			public void mouseReleased(MouseEvent arg0) {
//				// TODO Auto-generated method stub
//				
//			}
//		});
		
		cbDAP.addItemListener(new ItemListener() {

			@Override
			public void itemStateChanged(ItemEvent arg0) {
				DAPSpec dap = (DAPSpec) cbDAP.getSelectedItem();
				alternatives.copy(z3analyzer.DBMSList);
				
				if(dap.hasDurable) {
					//remove the dbms that do not support durability
					//int size = alternatives.DBMSSpecList.size(); 
					for(DBMSSpec dbms : z3analyzer.DBMSList.DBMSList) {
						int index = alternatives.findIndexByName(dbms.name);
						if(index >=0 && !dbms.hasDurability) {
							alternatives.removeElementAt(index);
						}
					}
				}
				if(dap.hasRealTime) {
					//remove the dbms that do not support timeliness
					//int size = z3analyzer.DBMSList.DBMSSpecList; 
					for(DBMSSpec dbms : z3analyzer.DBMSList.DBMSList) {
						if(!(dbms.hasTimeliness)) {
							int index = alternatives.findIndexByName(dbms.name);
							if(index >=0 ) {
								alternatives.removeElementAt(index);
							}
						}
					}
				}
				
			}
		});
		
		cbDBMS.addItemListener(new ItemListener() {

			@Override
			public void itemStateChanged(ItemEvent arg0) {
				// TODO Auto-generated method stub
				if(arg0.getStateChange() == ItemEvent.SELECTED) {
					DBMSSpec dbms = (DBMSSpec) cbDBMS.getSelectedItem();
					DAPSpec dap = (DAPSpec) cbDAP.getSelectedItem();
					ArrayList<String> rawdata = new ArrayList<String>();
					
					//if(dbms==null) return;
					textArea.setText("");
					textArea.append("Assuming each data object has an entity in database, the tables in " + dbms.name + " are : \n");
					String datamodel = "";
					for(RDTypeSpec rdt : dap.rawdatatypelist) {
						for(RawDataSpec rd : rdt.rawdatalist) {
							rawdata.add(dap.name+"_"+rdt.name+"_"+rd.name);
							datamodel = datamodel + dap.name+"_"+rdt.name+"_"+rd.name + ", ";
						}
					}
					textArea.append("Raw data tables: ");
					textArea.append(datamodel.substring(0, datamodel.length()-2));
					textArea.append("\nAggregated data table: ");
					textArea.append(dap.name + "_" + dap.aggdata.name + "\n");
					

					textArea.append("\nQueries for DAP: \n");
					String query = "";
					for(String rdn : rawdata) {
						query = query + "Select value from " + rdn + "\n";
					}
					textArea.append(query);
					textArea.append("Compute aggregated_data_value = " +  dap.name + "_" + dap.af + "(");

					for(int i=0;i<rawdata.size();i++) {
						textArea.append(rawdata.get(i));
						if(i<rawdata.size()-1) 
							textArea.append(", ");
					}
					textArea.append(")\n");
					query = "Insert into " + dap.name + "_" + dap.aggdata.name + " values (aggregated_data_value)";
					textArea.append(query);
				
				}
			}
			
		});

//		cbDBMS.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent e) {
//				DBMSSpec dbms = (DBMSSpec) cbDBMS.getSelectedItem();
//				DAPSpec dap = (DAPSpec) cbDAP.getSelectedItem();
//				ArrayList<String> rawdata = new ArrayList<String>();
//				
//				//if(dbms==null) return;
//				textArea.setText("");
//				textArea.append("Assuming each data object has an entity in database, the tables in " + dbms.name + " are : \n");
//				String datamodel = "";
//				for(RDTypeSpec rdt : dap.rawdatatypelist) {
//					for(RawDataSpec rd : rdt.rawdatalist) {
//						rawdata.add(dap.name+"_"+rdt.name+"_"+rd.name);
//						datamodel = datamodel + dap.name+"_"+rdt.name+"_"+rd.name + ", ";
//					}
//				}
//				textArea.append("Raw data tables: ");
//				textArea.append(datamodel.substring(0, datamodel.length()-2));
//				textArea.append("\nAggregated data table: ");
//				textArea.append(dap.name + "_" + dap.aggdata + "\n");
//				
//
//				textArea.append("\nQueries for DAP: \n");
//				String query = "";
//				for(String rdn : rawdata) {
//					query = query + "Select value from " + rdn + "\n";
//				}
//				textArea.append(query);
//				textArea.append("Compute aggregated_data_value = " +  dap.name + "_" + dap.af + "(");
//
//				for(int i=0;i<rawdata.size();i++) {
//					textArea.append(rawdata.get(i));
//					if(i<rawdata.size()-1) 
//						textArea.append(", ");
//				}
//				textArea.append(")\n");
//				query = "Insert into " + dap.name + "_" + dap.aggdata + " values (aggregated_data_value)";
//				textArea.append(query);
//			}
//			
//		});
	}
}
