package aggtool.ui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import com.microsoft.z3.BoolExpr;

import aggtool.z3.Z3Analyzer;

public class NewRulePanel extends JPanel {
	public JButton btAddRule = new JButton("Import Customized Constraints");
	public JButton btClear = new JButton("Clear Customized Constraints");
	public JFileChooser fc = new JFileChooser();
	public Z3Analyzer z3analyzer;
	public JTextArea txtRules = new JTextArea();

	public NewRulePanel(Z3Analyzer z) {
		super();
		z3analyzer = z;
		initialize();

	}

	void initialize() {
		this.setLayout(new BorderLayout());

		JPanel pButton = new JPanel(new FlowLayout(FlowLayout.LEFT));
		pButton.add(btAddRule);
		pButton.add(btClear);

		this.add(pButton, BorderLayout.NORTH);

		JScrollPane scrollPane = new JScrollPane(txtRules);

		this.add(scrollPane, BorderLayout.CENTER);
		implementRules();
	}

	void implementRules() {

		btAddRule.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				z3analyzer.CustomizedConstraints.clear();
				int returnVal = fc.showOpenDialog(NewRulePanel.this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File file = fc.getSelectedFile();
					// This is where a real application would open the file.
					System.out.println("Opening: " + file.getAbsolutePath()
							+ ".");
					addRulesFromFile(file.getAbsolutePath());
				}
			}
		});
		
		btClear.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				z3analyzer.CustomizedConstraints.clear();
				txtRules.setText("");
			}
		});


	}

	void addRulesFromFile(String path) {
		BoolExpr a;
		try {
			a = z3analyzer.z3context.parseSMTLIB2File(path, null, null,
				null, null);
			if(a==null) {
				txtRules.setText("Cannot parse the selected SMT2 file: " + path);
				return;
			}  
			txtRules.setText("Importing constraints from " + path + ":\n");
			z3analyzer.CustomizedConstraints.add(a);
			for(BoolExpr e : z3analyzer.CustomizedConstraints) {

				txtRules.append(e.toString() + "\n");
			}
		} catch (Exception e){
			txtRules.setText("Cannot parse the selected SMT2 file: " + path + ".\n" + e.getMessage());
			return;
		}
	}
}
