package aggtool.ui_old;

import java.awt.Dimension;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreeSelectionModel;

import aggtool.metadata.AggNode;
import aggtool.metadata.Taxonomy;

public class DataAggToolFrame extends JFrame {
	private AggNode AGGTREE;
	public DataAggToolFrame() {
		super();
		AGGTREE = Taxonomy.getTaxonomy();
		
		createUI();

		this.setVisible(true);

        this.setDefaultCloseOperation(EXIT_ON_CLOSE);
	}
	
	private void createUI() {
		CustomizedNode root = new CustomizedNode(AGGTREE.name);
		transformTree(AGGTREE, root);
		Taxonomy.printNode(AGGTREE);
		JTree tree = new JTree(root);
		
		CustomizedTreeRenderer renderer = new CustomizedTreeRenderer();
	    tree.setCellRenderer(renderer);

	    tree.setCellEditor(new CustomizedTreeCellEditor(tree));
	    tree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
	    tree.setEditable(true);
	    for (int i = 0; i < tree.getRowCount(); i++) {
	    	tree.expandRow(i);
	    }
		JScrollPane treeview = new JScrollPane(tree, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		Dimension minimumSize = new Dimension(100, 50);
		treeview.setMinimumSize(minimumSize);
		
		this.add(treeview);
		this.setSize(500, 500);
	}
	
	public void transformTree(AggNode anode, CustomizedNode jnode) {
		if(anode.isGroup) {
				GroupNode gnode = new GroupNode(anode.type);
				//jnode.add(gnode);
				for(AggNode member: anode.groupmembers) {
					MemberNode mnode = new MemberNode(member.name);
					mnode.copyAggNode(member);
					mnode.setGroup(gnode);
					gnode.addMember(mnode);
					jnode.add(mnode);
					if(member.children!=null) {			
						for(AggNode child: member.children) {
							transformTree(child, mnode);
						}
					} 
				}
		}
		else {
			CustomizedNode node = new CustomizedNode(anode.name);
			node.copyAggNode(anode);
			jnode.add(node);
			
			if(anode.children!=null) {			
				for(AggNode child: anode.children) {
					transformTree(child, node);
				}
			} 
		}
		
	}

	public static void main(String[] args) {
		// TODO Auto-generated method stub
		SwingUtilities.invokeLater(new Runnable(){
            @Override public void run(){
        		DataAggToolFrame frame = new DataAggToolFrame();
            }
        });
	}

}
