package aggtool.z3;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;

import aggtool.metadata.DAPFormulas;
import aggtool.metadata.DAPSpec;
import aggtool.metadata.DAPSpecList;
import aggtool.metadata.DBMSFormulas;
import aggtool.metadata.DBMSSpecList;
import aggtool.ui.DAPPanel;
import aggtool.ui.InterDAPDependencyPanel;
import aggtool.ui.RDTypePanel;
import aggtool.ui.RawDataPanel;

import com.microsoft.z3.AST;
import com.microsoft.z3.BoolExpr;
import com.microsoft.z3.Context;
import com.microsoft.z3.Expr;
import com.microsoft.z3.Solver;
import com.microsoft.z3.Status;
import com.microsoft.z3.Symbol;

public class Z3Analyzer {
	public Context z3context = new Context(); 
	public ArrayList<DAPFormulas> DAPFormulaList = new ArrayList<DAPFormulas>();
	public ArrayList<DBMSFormulas> DBMSFormulaList = new ArrayList<DBMSFormulas>();
	public ArrayList<BoolExpr> IntraDAPConstraints = new ArrayList<BoolExpr>();
	public ArrayList<BoolExpr> InterDAPConstraints = new ArrayList<BoolExpr>();
	public ArrayList<BoolExpr> DBMSConstraints = new ArrayList<BoolExpr>();
	public ArrayList<BoolExpr> CustomizedConstraints = new ArrayList<BoolExpr>();
	public DAPSpecList DAPList = new DAPSpecList();
	public DBMSSpecList DBMSList = new DBMSSpecList();
	
	public void addIntraDAPRules(ArrayList<DAPPanel> dappanels) {
		IntraDAPConstraints.clear();
		for(DAPPanel dap : dappanels) {
			String DAPName = dap.txtName.getText();
			String ADName = dap.pAggregatedData.txtName.getText();
			for(RDTypePanel rdt : dap.rdtypepanels) {
				String RDType = rdt.txtName.getText();
					for(RawDataPanel rd : rdt.rdpanels) {
						String RD = rd.txtName.getText();
						Z3Utils.createIntraDAPFormulas(this, DAPName, ADName, RDType, RD);
					}
			}		
		}
	}

	public void addInterDAPRules(ArrayList<InterDAPDependencyPanel> interdappanels) {
		InterDAPConstraints.clear();
		for(InterDAPDependencyPanel panel : interdappanels) {

			if(panel.cbDAPAggData.getSelectedItem()==null 
					|| panel.cbDependentDAP.getSelectedItem()==null
					|| panel.cbDependentDAPRDType.getSelectedItem()==null 
					|| panel.cbDependentDAPRawData.getSelectedItem()==null) continue;
			String DAPName1 = panel.cbDAPAggData.getSelectedItem().toString();
			DAPSpec dap1 = DAPList.getByName(DAPName1);
			String AD = dap1.aggdata.name;

			String DAPName2 = panel.cbDependentDAP.getSelectedItem().toString();
			//DAPSpec dap2 = DAPList.getByName(DAPName2);
			String RDT = panel.cbDependentDAPRDType.getSelectedItem().toString();
			String RD = panel.cbDependentDAPRawData.getSelectedItem().toString();
			
			Z3Utils.createInterDAPFormula(this, DAPName1, AD, DAPName2, RDT, RD);
		}
	}
	
	
	public static void main(String[] args) {
		//System.loadLibrary("libz3java.dll");
		//System.load("C:\\Users\\sci01\\Documents\\Tools\\z3-4.5.0-x64-win\\bin\\libz3java.dll");
		smt2FileTest("test.txt");
//	    Context c = new Context();
//	    Symbol x = c.mkSymbol("x");
//	    Symbol y = c.mkSymbol("y");
//	    BoolExpr bx = c.mkBoolConst("x");
//	    BoolExpr by = c.mkBoolConst("y");
//	    BoolExpr xandy = c.mkAnd(bx, by);
//	    BoolExpr notxandy = c.mkNot(xandy);
//	    BoolExpr notx = c.mkNot(bx);
//	    BoolExpr noty = c.mkNot(by);;
//	    BoolExpr notxornoty = c.mkOr(notx, noty);
//	    BoolExpr equal = c.mkEq(notxandy, notxornoty);
//
//	    
//	    Solver solver = c.mkSolver();
//	    solver.add(equal);
//
//		Status status = solver.check();
//		if (status == Status.SATISFIABLE)
//            System.out.println( solver.toString());
//        else
//        	System.out.println( "euqation wrong");
//	    //expr conjecture = !(x && y) == (!x || !y);
//		
//		unsatCoreAndProofExample(c);
	}
	
	
	 public static void unsatCoreAndProofExample(Context ctx)
	    {
	        System.out.println("UnsatCoreAndProofExample");

	        Solver solver = ctx.mkSolver();

	        BoolExpr pa = ctx.mkBoolConst("PredA");
	        BoolExpr pb = ctx.mkBoolConst("PredB");
	        BoolExpr pc = ctx.mkBoolConst("PredC");
	        BoolExpr pd = ctx.mkBoolConst("PredD");
	        BoolExpr p1 = ctx.mkBoolConst("P1");
	        BoolExpr p2 = ctx.mkBoolConst("P2");
	        BoolExpr p3 = ctx.mkBoolConst("P3");
	        BoolExpr p4 = ctx.mkBoolConst("P4");
	       
	        ArrayList<BoolExpr> assump = new ArrayList<BoolExpr> ();
	        assump.add(ctx.mkNot(p1));
	        assump.add(ctx.mkNot(p2));
	        assump.add(ctx.mkNot(p3));
	        assump.add(ctx.mkNot(p4));
	        BoolExpr[] assumarray = assump.toArray(new BoolExpr[0]);

	        BoolExpr[] assumptions = new BoolExpr[] { ctx.mkNot(p1), ctx.mkNot(p2),
	                ctx.mkNot(p3), ctx.mkNot(p4) };
	        BoolExpr f1 = ctx.mkAnd(pa, pb, pc);
	        BoolExpr f2 = ctx.mkAnd(pa, ctx.mkNot(pb), pc);
	        BoolExpr f3 = ctx.mkOr(ctx.mkNot(pa), ctx.mkNot(pc));
	        BoolExpr f4 = pd;

	        solver.add(ctx.mkOr(f1, p1));
	        solver.add(ctx.mkOr(f2, p2));
	        solver.add(ctx.mkOr(f3, p3));
	        solver.add(ctx.mkOr(f4, p4));
	        Status result = solver.check(assumarray);

	        if (result == Status.UNSATISFIABLE)
	        {
	            System.out.println("unsat");
	            //System.out.println("proof: " + solver.getProof());
	            System.out.println("core: ");
	            for (Expr c : solver.getUnsatCore())
	            {
	                System.out.println(c);
	            }
	        }
	    }
	 
	 
	 static void smt2FileTest(String filename)
	    {
	        Date before = new Date();

	        System.out.println("SMT2 File test ");
	        System.gc();

	        {
	            HashMap<String, String> cfg = new HashMap<String, String>();
	            cfg.put("model", "true");
	            Context ctx = new Context();
	            BoolExpr a = ctx.parseSMTLIB2File(filename, null, null, null, null);

	            long t_diff = ((new Date()).getTime() - before.getTime()) / 1000;

	            System.out.println("SMT2 file read time: " + t_diff + " sec");

	            // Iterate over the formula.

	            LinkedList<Expr> q = new LinkedList<Expr>();
	            q.add(a);
	            System.out.println(a.toString());
	            int cnt = 0;
	            while (q.size() > 0)
	            {
	                AST cur = (AST) q.removeFirst();
	                cnt++;

	                if (cur.getClass() == Expr.class)
	                    if (!(cur.isVar()))
	                        for (Expr c : ((Expr) cur).getArgs())
	                            q.add(c);
	            }
	            System.out.println(cnt + " ASTs");
	        }

	        long t_diff = ((new Date()).getTime() - before.getTime()) / 1000;
	        System.out.println("SMT2 file test took " + t_diff + " sec");
	    }

	    
}
