package aggtool.z3;

import com.microsoft.z3.BoolExpr;
import com.microsoft.z3.Context;

public class Z3Utils {
	public static BoolExpr createBooleanFormula(Context c, String variable, boolean selected) {
		if(selected) 
			return c.mkBoolConst(variable);
		else
			return c.mkNot(c.mkBoolConst(variable));
	}
	
	public static void createIntraDAPFormulas(Z3Analyzer z, String DAPName, String AD, String RDType, String RD) {
		Context c = z.z3context;
		String ad_prefix = DAPName + "_" + AD + "_RealTime";
		BoolExpr ad_rt_h = c.mkBoolConst(ad_prefix + "_Hard");
		BoolExpr ad_rt_f = c.mkBoolConst(ad_prefix + "_Firm");
		BoolExpr ad_rt_s = c.mkBoolConst(ad_prefix + "_Soft");
		
		String rd_prefix = DAPName + "_" + RDType + "_" + RD;
		BoolExpr rd_rt = c.mkBoolConst(rd_prefix + "_RealTime");
		BoolExpr rd_rt_h = c.mkBoolConst(rd_prefix + "_RealTime_Hard");
		BoolExpr rd_rt_f = c.mkBoolConst(rd_prefix + "_RealTime_Firm");
		BoolExpr rd_rt_s = c.mkBoolConst(rd_prefix + "_RealTime_Soft");
		
		BoolExpr dap_rt_h = c.mkBoolConst(DAPName + "_RealTimeProcess_Hard");
		BoolExpr dap_rt_f = c.mkBoolConst(DAPName + "_RealTimeProcess_Firm");
		
		//1. The real-time strictness level of the raw data must be higher than or equal to the real-time strictness level of the aggregated data.
		//AggregatedData.hard && RawData.RealTime => RawData.hard
		z.IntraDAPConstraints.add(c.mkImplies(c.mkAnd(ad_rt_h, rd_rt), rd_rt_h));

		//AggregatedData.firm && RawData.RealTime => RawData.hard || RawData.firm
		z.IntraDAPConstraints.add(c.mkImplies(c.mkAnd(ad_rt_f, rd_rt), c.mkOr(rd_rt_h, rd_rt_f)));
		
		//2. The strictness level w.r.t. timeliness of the entire DAP must be higher than or equal to the real-time strictness level of the aggregated data.
		//AggregatedData.hard  => DAP.RealTimeProcess.hard
		z.IntraDAPConstraints.add(c.mkImplies(ad_rt_h, dap_rt_h));

		//AggregatedData.firm  => DAP.RealTimeProcess.hard || DAP.RealTimeProcess.firm
		z.IntraDAPConstraints.add(c.mkImplies(ad_rt_h, c.mkOr(dap_rt_h, dap_rt_f)));
	}
	
	//DAP1.AggregatedData is the raw data of DAP2. If DAP2 requires the raw data to be real-time, then the DAP1's aggregated data must also be real-time. 
	public static void createInterDAPFormula(Z3Analyzer z, String DAPName1, String AD, String DAPName2, String RDType, String RD) {
		Context c = z.z3context;
		String ad_prefix = DAPName1 + "_" + AD + "_RealTime";
		BoolExpr ad_rt = c.mkBoolConst(ad_prefix);
		BoolExpr ad_rt_h = c.mkBoolConst(ad_prefix + "_Hard");
		BoolExpr ad_rt_f = c.mkBoolConst(ad_prefix + "_Firm");
		BoolExpr ad_rt_s = c.mkBoolConst(ad_prefix + "_Soft");
		
		String rd_prefix = DAPName2 + "_" + RDType + "_" + RD;
		BoolExpr rd_rt = c.mkBoolConst(rd_prefix + "_RealTime");
		BoolExpr rd_rt_h = c.mkBoolConst(rd_prefix + "_RealTime_Hard");
		BoolExpr rd_rt_f = c.mkBoolConst(rd_prefix + "_RealTime_Firm");
		BoolExpr rd_rt_s = c.mkBoolConst(rd_prefix + "_RealTime_Soft");


		// The real-time strictness level of the aggregated data of the providing DAP 
		//   must be higher than or equal to the real-time strictness level of the raw data that uses this aggregated data.
		// DAP1.AggregatedData must be real-time, because DAP1 requires the data to reflect the timely state.
		//DAP2.RawData.RealTime.hard => DAP1.AggregatedData.RealTime.hard
				z.InterDAPConstraints.add(c.mkImplies(rd_rt_h, ad_rt_h));
		//DAP2.RawData.RealTime.firm => DAP1.AggregatedData.RealTime.hard || DAP1.AggregatedData.RealTime.firm
				z.InterDAPConstraints.add(c.mkImplies(rd_rt_f, c.mkOr(ad_rt_h, ad_rt_f)));
		//DAP2.RawData.RealTime.soft => DAP1.AggregatedData.RealTime.hard || DAP1.AggregatedData.RealTime.firm || DAP1.AggregatedData.RealTime.soft
				z.InterDAPConstraints.add(c.mkImplies(rd_rt_s, c.mkOr(ad_rt_h, ad_rt_f, ad_rt_s)));
	}
}
